import importlib.metadata


class Project:
    @classmethod
    def name(cls):
        return __package__ or __name__

    @classmethod
    def version(cls):
        try:
            return importlib.metadata.version(cls.name())
        except importlib.metadata.PackageNotFoundError:
            return 'dev'



import os
import shlex
import shutil
import logging
from typing import List


def execute_graphviz(graphviz_code: str, arguments: List[str]) -> str:
    """
    Executes a graphviz command
    """
    executable = shutil.which('dot') or shutil.which('graphviz')

    if not executable:
        raise ModuleNotFoundError(f'graphviz / dot executable not found! '
                                  f'Please install graphviz and make sure you have it in the PATH variable.')

    arg = " ".join(arguments)
    cmd = f'echo {shlex.quote(graphviz_code)} | {executable} {arg}'
    logging.debug(f"Executing in command line: {cmd}")
    os.system(cmd)




import sys
import re
import logging
from typing import List

py_regex = r'{{( |[= ]|\n)((?!{{ |{{= | }})[\s\S])+( |\n)}}'


def convert(python_graphviz: str, arguments: List[str]) -> str:
    """
    Convert python graphviz to graphviz
    """

    for arg in arguments:
        exec(arg)

    python_codes = re.finditer(py_regex, python_graphviz)
    position_correction = 0

    for code in python_codes:
        expression = code.group()
        variable = expression[2] == '='

        py_code = expression[4 if variable else 3:-3]

        try:
            if variable:
                output = eval(py_code)

                has_decimal_places = type(output) == float

                if not has_decimal_places:
                    try:
                        import numpy as np
                        has_decimal_places = type(output) == np.float64
                    except ImportError:
                        pass

                if has_decimal_places:
                    r = 2
                    try:
                        r = round_digits
                    except NameError:
                        pass

                    num = float(output)
                    output = round(num, r)

                    logging.debug(f"Rounded {num} to output")

                output = str(output)
            else:
                exec(py_code)
                output = ""
        except Exception as ex:
            raise Exception(f"Error while processing: {expression}", ex)

        python_graphviz = f"{python_graphviz[:code.start() + position_correction]}{output}{python_graphviz[code.end() + position_correction:]}"

        position_correction = position_correction + (len(output) - len(expression))

        if variable:
            logging.debug(f"Executing with output: {expression} ⟶ {output}")
        else:
            logging.debug(f"Executing without output: {expression}")

    # Clear all variables
    sys.modules[__name__].__dict__.clear()

    return python_graphviz



import sys
import argparse
import logging
from pathlib import Path

from graphviz_py.graphviz import execute_graphviz
from graphviz_py.parser import convert
from graphviz_py.project import Project

parser = argparse.ArgumentParser(description='graphviz-py diagram builder')
parser.add_argument('-v', '--version', action='version', version=f'{Project.name()} version: {Project.version()}')
parser.add_argument('-d', '--debug', action='store_true', help='show debug information')
parser.add_argument('-a', '--argument', action='append')
parser.add_argument('files', type=Path, nargs=argparse.ZERO_OR_MORE, help='the paths to the graphviz-py files')

args, unknown = parser.parse_known_args()


def main(_=None):
    """Process command line arguments."""

    if args.debug:
        logging.getLogger().setLevel(logging.DEBUG)

    file_contents = []
    arguments = []

    if args.argument:
        arguments = args.argument

    if args.files:
        for file in args.files:
            with open(file) as f:
                file_contents.append(f.read())

    if len(file_contents) == 0:
        file_contents.append(sys.stdin.read())

    for c in file_contents:
        graphviz_content = convert(c, arguments)
        execute_graphviz(graphviz_content, unknown)





# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['graphviz_py']

package_data = \
{'': ['*']}

entry_points = \
{'console_scripts': ['graphviz-py = graphviz_py.cli:main']}

setup_kwargs = {
    'name': 'graphviz-py',
    'version': '1.0.5',
    'description': 'Allows Python code execution inside of graphviz diagrams.',
    'long_description': '# graphviz-py\n[![package version](https://img.shields.io/pypi/v/graphviz-py?style=flat-square&color=%2300AA00)](https://pypi.org/project/graphviz-py/)\n[![py versions](https://img.shields.io/pypi/pyversions/graphviz-py?style=flat-square)](https://pypi.org/project/graphviz-py/)\n[![pypi](https://img.shields.io/github/workflow/status/Alwinator/graphviz-py/Publish%20to%20PyPi?label=Build%20%26%20Publish&style=flat-square)](https://github.com/Alwinator/graphviz-py/actions)\n[![license](https://img.shields.io/github/license/Alwinator/graphviz-py?style=flat-square&color=%23006699)](LICENSE)\n\nAllows Python code execution inside of [graphviz](https://graphviz.org/) diagrams\n\n## Example\n```dot\ngraph python_graph {\n{{\nimport math\n\nvalue = 0.5\nsin = math.sin(value)\ncos = math.cos(value)\n}}\n\nA [label="{{= value }}"];\nB [label="{{= sin }}"];\nC [label="{{= cos }}"];\n\nA -- B [headlabel="sin"];\nA -- C [headlabel="cos"];\n\n}\n```\n\n### Output\n![output](https://raw.githubusercontent.com/Alwinator/graphviz-py/main/assets/output_file.svg)\n\n## Installation\n```bash\npip install graphviz-py\n```\n\n**Important: Make sure graphviz is installed!** See [graphviz installation instructions](https://graphviz.org/download/).\n\n\n## Usage\n### Using files\n```bash\ngraphviz-py -Tsvg example/example.py.dot -o output.svg\ngraphviz-py -Tpng example/example.py.dot -o output.png\n```\n\n### Using stdin / pipes\n[comment]: <> (echo \'digraph { node [fontname="Arial"]; edge [fontname="Arial"];  A -> B [label="  {{= 38 * 73 }}"] }\' | graphviz-py -Tsvg > output.svg)\n```bash\necho \'digraph { A -> B [label="{{= 38 * 73 }}"] }\' | graphviz-py -Tsvg > output.svg\n```\ngraphviz-py passes all unknown arguments to graphviz. So you can use all [graphviz arguments](https://graphviz.org/doc/info/command.html).\n\n### Output\n![output](https://raw.githubusercontent.com/Alwinator/graphviz-py/main/assets/output_pipe.svg)\n\n## Variables\n```bash\ngraphviz-py -Tsvg -a myvalue=5 example/variable_example.py.dot -o output.svg\n```\nHere we pass a variable called "myvalue" with the value 5\n\n### Output\n![output](https://raw.githubusercontent.com/Alwinator/graphviz-py/main/assets/output_variable.svg)\n\n## Security\nPlease keep in mind that graphviz-py executes all Python code in the diagram. So make sure that your diagrams dies not include harmful code.\n\n## Coming soon\n- Compartibility with asciidoctor-diagram ([Status: Implemented & Approved, waiting for merging](https://github.com/asciidoctor/asciidoctor-diagram/pull/379))\n\n## Arguments\n```bash\n# graphviz-py --help\nusage: graphviz-py [-h] [-v] [-d] [-a ARGUMENT] [files [files ...]]\n\ngraphviz-py diagram builder\n\npositional arguments:\n  files                 the paths to the graphviz-py files\n\noptional arguments:\n  -h, --help            show this help message and exit\n  -v, --version         show program\'s version number and exit\n  -d, --debug           show debug information\n  -a ARGUMENT, --argument ARGUMENT\n```\n',
    'author': 'Alwin Schuster',
    'author_email': 'contact@alwinschuster.at',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/Alwinator/graphviz-py',
    'packages': packages,
    'package_data': package_data,
    'entry_points': entry_points,
    'python_requires': '>=3.8.0',
}


setup(**setup_kwargs)
